/* $Id: app_enumquery.c,v 1.5 2004/04/27 20:42:43 root Exp root $ */
/*
 * EnumQuery: Asterisk module for E.164 Query
 *
 * Author: Karl Brose, khb@brose.com
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <pthread.h>

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/options.h>
#include <asterisk/config.h>
#include <asterisk/module.h>
#include <asterisk/enum.h>

#define STRcopy(to,fro) strncpy(to, fro, sizeof(to)-1)


#define CONFIG_FILE	"enum.conf"

static char *tdesc	= "ENUM Query";
static char *app	= "EnumQuery";
static char *synopsis	= "Lookup number in e164 ENUM";
static char *modversion	= "$Id: app_enumquery.c,v 1.5 2004/04/27 20:42:43 root Exp root $";
static char *descrip = 
"  EnumQuery(exten):  Looks up an extension via ENUM and sets the variables\n"
"'ENUM', 'ENUMTECH', 'ENUMDEST', 'ENUMURI', 'ENUMNUM', 'ENUMDOMAIN'.\n"
"For VoIP URIs ENUM will look like 'TECHNOLOGY/URI' with the appropriate technology.\n"
"ENUMURI returns the  URI portion, and ENUMTECH just the technology.  ENUMDEST is just the\n"
"destination number/user and domain, ENUMNUM is just the number, and ENUMDOMAIN is the\n"
"domain portion of the URI. EnumLookup returns -1 on hangup or error, or 0 on completion,\n"
"regardless of whether the lookup was successful. A succesful query will result in normal\n"
"priority handling. If the lookup was *not* successful and there exists a priority n + 101,\n"
"then that priority will be taken next.\n"
"The technologies received from the queries are normally passed on to the PBX,\n"
"alternately they can be mapped to specific Asterisk drivers in the configuration\n"
"file (technology => driver) (10 max).\n"
;


#define MAX_TECH_LEN	10
#define MAX_NTECH	10
static int ntech = 0;
static struct techno{
	char tech[MAX_TECH_LEN];
	char driver[MAX_TECH_LEN];
} techdrv[MAX_NTECH] =			/* until we expand this */
	{{"",""},{"",""},{"",""},{"",""},{"",""},{"",""},{"",""},{"",""},{"",""},{"",""}};



STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int exec_enumquery(struct ast_channel *chan, void *data)
{
	struct localuser *u;
	int res=0;
	char tech[MAX_TECH_LEN];
	char uri[256];
	char tmp[256];
	char *driver="";
	char *dest="";
	char *c,*t;
	int	i,k,n;

	if (!data || !strlen(data)) {
		ast_log(LOG_WARNING, "EnumLookup requires an argument (extension)\n");
		if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 101, chan->callerid))
			chan->priority += 100;
		return -1;
	}

	LOCAL_USER_ADD(u);

	res = ast_get_enum(chan, data, uri, sizeof(uri), tech, sizeof(tech));

	LOCAL_USER_REMOVE(u);

	if (option_verbose > 1)
		ast_verbose(VERBOSE_PREFIX_2 "ENUM LOOKUP: context=%s, data=%s ===> res=%0d  tech=%s  uri=%s\n", chan->context, (char*)data, res, tech, uri);

	if(!res) {
		if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 101, chan->callerid))
			chan->priority += 100;
		ast_log(LOG_WARNING, "ENUM LOOKUP failed:  res=%d\n",  res);
		return 0;
	}
	
	k=-1;
	dest = uri;
	n=strlen(tech);
	t=strchr(uri,':');
	if ( t && !strncasecmp(uri,tech,n))
		dest = t + 1;

	for(i=0;i<ntech;i++){		/* map to driver */
		if (!strcasecmp(tech, techdrv[i].tech)) {
			driver=techdrv[i].driver;	/* found a match */
			k=i;
			break;
		}
	}

	if( (k<0) && (n>0)){			/* no explicit mapping in config */
			driver = tech;		/* pass received technology */
#if 0
			/* alternative:  we could block all not configured technologies: */
			ast_log(LOG_WARNING, "ENUM LOOKUP: Don't know how to handle technology '%s'\n", tech);
			res = 0;
#endif
	}


	/* special cases processing */

	if (!strcasecmp(tech, "H323")) {
		t = strchr(uri,';');
		if (t) *t = 0;		/* stop at ';' */
	}
	if (!strcasecmp(tech, "TEL")) {
		c = dest;
		if (c[0] != '+') {
			ast_log(LOG_WARNING, "tel: uri must start with a \"+\" (got '%s')\n", c);
			res = 0;
		} else {		/* copy over the number, skipping all non-digits and stop at ; or NULL */
			t = tmp;	
			while( *c && (*c != ';') && (t - tmp < (sizeof(tmp) - 1))) {
				if (isdigit(*c)) *t++ = *c;
				c++;
			}
			*t = 0;
			dest = tmp;
			if (option_verbose > 1)
				ast_verbose(VERBOSE_PREFIX_2 "ENUM Lookup: ENUM set to \"%s\" for TEL\n", tmp);
			res = 2;
		}
	}

	if (res > 0) {
		pbx_builtin_setvar_helper(chan, "ENUMURI",  uri);
		pbx_builtin_setvar_helper(chan, "ENUMDEST", dest);
		pbx_builtin_setvar_helper(chan, "ENUMTECH", driver);
		if (res==2){	/* TEL */
				pbx_builtin_setvar_helper(chan, "ENUMNUM", dest);
				pbx_builtin_setvar_helper(chan, "ENUM", tmp);
		} else {
			snprintf(tmp, sizeof(tmp), "%s/%s", driver, dest);
			t = strchr(dest,'@');
			if (t) {
				*t = '\0';
				pbx_builtin_setvar_helper(chan, "ENUMNUM", dest);
				pbx_builtin_setvar_helper(chan, "ENUMDOMAIN", ++t);
			}
		}
		res = 0;
	} else {
		if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 101, chan->callerid))
			chan->priority += 100;
	}
	return res;
}

static int load_config(void)
{
	struct ast_config *cfg;
	struct ast_variable *v = NULL;

	cfg = ast_load(CONFIG_FILE);
	if (!cfg) {
		ast_log(LOG_WARNING, "Missing configuration file %s, using internal defaults\n", CONFIG_FILE);
		return 0;
	}

	ntech = 0;
	v = ast_variable_browse(cfg, "general");
	while (v){
		if (strcasecmp(v->name, "search")) {	/* skip the search lines */
			if (strlen(v->value) > 0){	/* all others are tech definitions */
				ntech++;
				if (ntech>MAX_NTECH){
					ast_log(LOG_WARNING, "ENUM LOOKUP: Can only handle %d technology mappings\n", MAX_NTECH);
					ntech=MAX_NTECH;
					break;
				}
				STRcopy(techdrv[ntech].tech, v->name);
				STRcopy(techdrv[ntech].driver, v->value);
				if (option_verbose > 1){
					ast_verbose(VERBOSE_PREFIX_2 "ENUM LOOKUP: Configured technology mapping %s => %s\n", v->name, v->value);
				}
			}
		}
	v = v->next;
	}

	ast_destroy(cfg);

	return 0;
}


int load_module(void)
{
	int res;

	ast_log(LOG_DEBUG, "%s/%s\n", app, modversion);
	res = ast_register_application(app, exec_enumquery, synopsis, descrip);
	if (res)
		return res;

	res=load_config();

	return res;
}

int reload(void)
{
	return(load_config());
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}

#if 0
char *version(void)
{
	return modversion;
}
#endif
/*
; enum.conf
;

[general]

; The search list for domains may be customized.  Domains are searched
; in the order they are listed here.

search => priv.brose.com
search => e164.arpa


; designate the Asterisk channel driver that services each enum technology
; enum technology => channel prefix
;
sip=SIP10
iax2=IAX2
iax=IAX
h323=H323

*/
/*
 * $Log: app_enumquery.c,v $
 * Revision 1.5  2004/04/27 20:42:43  root
 * New configuration file parser. Accepts up to 10 technology/driver mappings.
 * Without mappings, the technology in the URI is passed to PBX.
 * Need to decide what to do when prio 51 doesn't exist for TEL
 * debug messages are printed when verbose is on
 *
 * Revision 1.5  2004/04/19 22:16:06  root
 * *** empty log message ***
 *
 * Revision 1.4  2004/04/15 17:14:05  root
 * changed log message priorities, code cleanup
 *
 * Revision 1.3  2004/04/15 08:07:58  asterisk
 * Added two additional variable returned, ENUMNUM and ENUMDOMAIN
 *
 */
